﻿using LawFirm;
using LawFirmContracts.BindingModels;
using LawFirmContracts.BusinessLogicContracts;
using LawFirmContracts.SearchModels;
using LawFirmDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace LawFirmView
{
    public partial class FormDocument : Form
    {
        private readonly ILogger _logger;
        private readonly IDocumentLogic _logic;
        private int? _id;
        private Dictionary<int, (IBlankModel, int)> _documentBlanks;
        public int Id { set { _id = value; } }
        public FormDocument(ILogger<FormDocument> logger, IDocumentLogic logic)
        {
            InitializeComponent();
            _logger= logger;
            _logic= logic;
            _documentBlanks = new Dictionary<int, (IBlankModel, int)>();
        }

        private void FormDocument_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка документа");
                try
                {
                    var view = _logic.ReadElement(new DocumentSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        textBoxName.Text = view.DocumentName;
                        textBoxPrice.Text = view.Price.ToString();
                        _documentBlanks = view.DocumentBlanks ?? new Dictionary<int, (IBlankModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки документа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }
        }
        private void LoadData()
        {
            _logger.LogInformation("Загрузка бланков документа");
            try
            {
                if (_documentBlanks != null)
                {
                    dataGridView.Rows.Clear();
                    foreach (var pc in _documentBlanks)
                    {
                        dataGridView.Rows.Add(new object[] 
                        { 
                          pc.Key,
                          pc.Value.Item1.BlankName, 
                          pc.Value.Item2
                        }
                        );
                    }
                    textBoxPrice.Text = CalcPrice().ToString();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки бланков документа");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
               MessageBoxIcon.Error);
            }
        }

        private double CalcPrice()
        {
            double price = 0;
            foreach (var elem in _documentBlanks)
            {
                price += ((elem.Value.Item1?.Cost ?? 0) * elem.Value.Item2);
            }
            return Math.Round(price * 1.1, 2);
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormDocumentBlank));
            if (service is FormDocumentBlank form)
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (form.BlankModel == null)
                    {
                        return;
                    }
                    _logger.LogInformation("Добавление нового бланка:{ BlankName}- { Count}", form.BlankModel.BlankName, form.Count);
                    if (_documentBlanks.ContainsKey(form.Id))
                    {
                        _documentBlanks[form.Id] = (form.BlankModel,form.Count);
                    }
                    else
                    {
                        _documentBlanks.Add(form.Id, (form.BlankModel,form.Count));
                    }
                    LoadData();
                }
            }
        }

        private void buttonEdit_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                var service =  Program.ServiceProvider?.GetService(typeof(FormDocumentBlank));
                if (service is FormDocumentBlank form)
                {
                    int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value);
                    form.Id = id;
                    form.Count = _documentBlanks[id].Item2;
                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        if (form.BlankModel == null)
                        {
                            return;
                        }
                        _logger.LogInformation("Изменение бланка:{ BlankName}- { Count}", form.BlankModel.BlankName, form.Count);
                        _documentBlanks[form.Id] = (form.BlankModel,form.Count);
                        LoadData();
                    }
                }
            }
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос",
               MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    try
                    {
                        _logger.LogInformation("Удаление бланка:{ BlankName}- { Count}", dataGridView.SelectedRows[0].Cells[1].Value);
                        _documentBlanks?.Remove(Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    LoadData();
                }
            }
        }

        private void buttonUpdate_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxPrice.Text))
            {
                MessageBox.Show("Заполните цену", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (_documentBlanks == null || _documentBlanks.Count == 0)
            {
                MessageBox.Show("Заполните бланки", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение документа");
            try
            {
                var model = new DocumentBindingModel
                {
                    Id = _id ?? 0,
                    DocumentName = textBoxName.Text,
                    Price = Convert.ToDouble(textBoxPrice.Text),
                    DocumentBlanks = _documentBlanks
                };
                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения документа");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void dataGridView_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {

        }
    }
}
