﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.BusinessLogicContracts;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StorageContracts;
using LawFirmContracts.ViewModels;
using LawFirmDataModels.Enums;
using LawFirmDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LawFirmBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IShopStorage _shopStorage;
        private readonly IShopLogic _shopLogic;
        private readonly IDocumentStorage _documentStorage;


        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IShopLogic shopLogic, IDocumentStorage documentStorage, IShopStorage shopStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _shopLogic = shopLogic;
            _documentStorage = documentStorage;
            _shopStorage = shopStorage;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Order status incorrect.");
                return false;
            }
            model.Status = OrderStatus.Принят;
            if (_orderStorage.Insert(model) == null)
            {
                model.Status = OrderStatus.Неизвестен;
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool StatusUpdate(OrderBindingModel rawModel, OrderStatus newStatus) 
        {
            var viewModel = _orderStorage.GetElement(new OrderSearchModel
            {
                Id = rawModel.Id
            });

            if (viewModel == null)
            {
                _logger.LogWarning("Order model not found");
                return false;
            }

            OrderBindingModel model = new OrderBindingModel
            {
                Id = viewModel.Id,
                DocumentId = viewModel.DocumentId,
                Status = viewModel.Status,
                DateCreate = viewModel.DateCreate,
                DateImplement = viewModel.DateImplement,
                Count = viewModel.Count,
                Sum = viewModel.Sum
            };

            CheckModel(model, false);
            if (model.Status + 1 != newStatus)
            {
                _logger.LogWarning("Status update to " + newStatus.ToString() +" operation failed. Order status incorrect.");
                return false;
            }

            if (newStatus == OrderStatus.Готов)
            {
                var document = _documentStorage.GetElement(new DocumentSearchModel() { Id = model.DocumentId});
                if (document == null)
                {
                    _logger.LogWarning("Status update to " + newStatus.ToString() + " operation failed. Document not found.");
                    return false;
                }
                if (CheckThenSupplyMany(document, model.Count) == false)
                {
                    _logger.LogWarning("Status update to " + newStatus.ToString() + " operation failed. Shop supply error.");
                    return false;
                }
            }

            model.Status = newStatus;
            if (model.Status == OrderStatus.Выдан) model.DateImplement = DateTime.Now;
            if (_orderStorage.Update(model) == null)
            {
                model.Status--;
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool CheckThenSupplyMany(IDocumentModel document, int count)
        {
            if (count <= 0)
            {
                _logger.LogWarning("Check then supply operation error. Document count < 0.");
                return false;
            }

            int freeSpace = 0;
            foreach (var shop in _shopStorage.GetFullList())
            {
                freeSpace += shop.MaxCountDocuments;
                foreach (var doc in shop.ShopDocuments)
                {
                    freeSpace -= doc.Value.Item2;
                }
            }

            if (freeSpace - count < 0)
            {
                _logger.LogWarning("Check then supply operation error. There's no place for new docs in shops.");
                return false;
            }

            foreach (var shop in _shopStorage.GetFullList())
            {
                freeSpace = shop.MaxCountDocuments;
                foreach (var doc in shop.ShopDocuments)
                {
                    freeSpace -= doc.Value.Item2;
                }
                if (freeSpace == 0)
                {
                    continue;
                }
                if (freeSpace - count >= 0)
                {
                    if (_shopLogic.SupplyDocuments(new() { Id = shop.Id }, document, count)) count = 0;
                    else
                    {
                        _logger.LogWarning("Supply error");
                        return false;
                    }
                }
                if (freeSpace - count < 0)
                {
                    if (_shopLogic.SupplyDocuments(new() { Id = shop.Id }, document, freeSpace)) count -= freeSpace;
                    else
                    {
                        _logger.LogWarning("Supply error");
                        return false;
                    }
                }
                if (count <= 0)
                {
                    return true;
                }
            }
            return false;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выполняется);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Готов);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выдан);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("Order. OrderID:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.DocumentId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор документа", nameof(model.DocumentId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество документов в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. OrderID:{Id}.Sum:{ Sum}. DocumentId: { DocumentId}", model.Id, model.Sum, model.DocumentId);
        }
    }
}
