﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.BusinessLogicContracts;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StorageContracts;
using LawFirmContracts.ViewModels;
using LawFirmDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace LawFirmBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Shop Name:{0}.ID:{1}", model.Name, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. Shop Name:{0}.ID:{1} ", model?.Name, model?.Id);

            var list = (model == null) ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool SupplyDocuments(ShopSearchModel model, IDocumentModel document, int count)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (document == null)
            {
                throw new ArgumentNullException(nameof(document));
            }

            if (count <= 0)
            {
                throw new ArgumentNullException("Count of documents in supply myst be more than 0", nameof(count));
            }

            var shopElement = _shopStorage.GetElement(model);
            if (shopElement == null) 
            {
                _logger.LogWarning("Required shop element not found in storage");
                return false;
            }
            _logger.LogInformation("Shop element found. ID: {0}, Name: {1}", shopElement.Id, shopElement.Name);

            var countDocs = 0;
            foreach (var doc in shopElement.ShopDocuments)
            {
                countDocs += doc.Value.Item2;
            }
            if (shopElement.MaxCountDocuments - countDocs >= count)
            {
                if (shopElement.ShopDocuments.TryGetValue(document.Id, out var sameDocument))
                {
                    shopElement.ShopDocuments[document.Id] = (document, sameDocument.Item2 + count);
                    _logger.LogInformation("Same document found by supply. Added {0} of {1} in {2} shop", count, document.DocumentName, shopElement.Name);
                }
                else
                {
                    shopElement.ShopDocuments[document.Id] = (document, count);
                    _logger.LogInformation("New document added by supply. Added {0} of {1} in {2} shop", count, document.DocumentName, shopElement.Name);
                }
                _shopStorage.Update(new()
                {
                    Id = shopElement.Id,
                    Name = shopElement.Name,
                    Adress = shopElement.Adress,
                    OpeningDate = shopElement.OpeningDate,
                    ShopDocuments = shopElement.ShopDocuments,
                    MaxCountDocuments = shopElement.MaxCountDocuments
                });
            }
            else
            {
                _logger.LogWarning("Required shop is overflowed");
                return false;
            }
            return true;
        }

        public bool SellDocument(IDocumentModel document, int count)
        {;
            return _shopStorage.SellDocument(document, count);
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия магазина!", nameof(model.Name));
            }
            if (model.MaxCountDocuments < 0)
            {
                throw new InvalidOperationException("Магазин с отрицательным количеством максимального количества документов!");
            }
            _logger.LogInformation("Shop. Name: {0}, Adress: {1}, ID: {2}", model.Name, model.Adress, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.Name
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
    }
}
